@@ -1,24 +1,51 @@
+import mimetypes
 import re
 import uuid
 
-from typing import List
+from os import path
+from requests import Response
+from typing import BinaryIO, List, Optional
 from urllib.parse import urlparse, urlencode, quote
 
-from toot import http, CLIENT_NAME, CLIENT_WEBSITE
-from toot.exceptions import AuthenticationError
-from toot.utils import str_bool, str_bool_nullable
+from toot import App, User, http, CLIENT_NAME, CLIENT_WEBSITE
+from toot.exceptions import AuthenticationError, ConsoleError
+from toot.utils import drop_empty_values, str_bool, str_bool_nullable
+
 
 SCOPES = 'read write follow'
 
 
-def _account_action(app, user, account, action):
+def find_account(app, user, account_name):
+    if not account_name:
+        raise ConsoleError("Empty account name given")
+
+    normalized_name = account_name.lstrip("@").lower()
+
+    # Strip @<instance_name> from accounts on the local instance. The `acct`
+    # field in account object contains the qualified name for users of other
+    # instances, but only the username for users of the local instance. This is
+    # required in order to match the account name below.
+    if "@" in normalized_name:
+        [username, instance] = normalized_name.split("@", maxsplit=1)
+        if instance == app.instance:
+            normalized_name = username
+
+    response = search(app, user, account_name, type="accounts", resolve=True)
+    for account in response.json()["accounts"]:
+        if account["acct"].lower() == normalized_name:
+            return account
+
+    raise ConsoleError("Account not found")
+
+
+def _account_action(app, user, account, action) -> Response:
     url = f"/api/v1/accounts/{account}/{action}"
-    return http.post(app, user, url).json()
+    return http.post(app, user, url)
 
 
-def _status_action(app, user, status_id, action, data=None):
+def _status_action(app, user, status_id, action, data=None) -> Response:
     url = f"/api/v1/statuses/{status_id}/{action}"
-    return http.post(app, user, url, data=data).json()
+    return http.post(app, user, url, data=data)
 
 
 def _tag_action(app, user, tag_name, action):
@@ -26,8 +53,8 @@ def _tag_action(app, user, tag_name, action):
     return http.post(app, user, url).json()
 
 
-def create_app(domain, scheme='https'):
-    url = f"{scheme}://{domain}/api/v1/apps"
+def create_app(base_url):
+    url = f"{base_url}/api/v1/apps"
 
     json = {
         'client_name': CLIENT_NAME,
@@ -85,10 +112,9 @@ def update_account(
     Update account credentials
     https://docs.joinmastodon.org/methods/accounts/#update_credentials
     """
-    files = {"avatar": avatar, "header": header}
-    files = {k: v for k, v in files.items() if v is not None}
+    files = drop_empty_values({"avatar": avatar, "header": header})
 
-    data = {
+    data = drop_empty_values({
         "bot": str_bool_nullable(bot),
         "discoverable": str_bool_nullable(discoverable),
         "display_name": display_name,
@@ -97,8 +123,7 @@ def update_account(
         "source[language]": language,
         "source[privacy]": privacy,
         "source[sensitive]": str_bool_nullable(sensitive),
-    }
-    data = {k: v for k, v in data.items() if v is not None}
+    })
 
     return http.patch(app, user, "/api/v1/accounts/update_credentials", files=files, data=data)
 
@@ -172,7 +197,11 @@ def post_status(
     language=None,
     scheduled_at=None,
     content_type=None,
-):
+    poll_options=None,
+    poll_expires_in=None,
+    poll_multiple=None,
+    poll_hide_totals=None,
+) -> Response:
     """
     Publish a new status.
     https://docs.joinmastodon.org/methods/statuses/#create
@@ -182,7 +211,9 @@ def post_status(
     # if the request is retried.
     headers = {"Idempotency-Key": uuid.uuid4().hex}
 
-    json = {
+    # Strip keys for which value is None
+    # Sending null values doesn't bother Mastodon, but it breaks Pleroma
+    data = drop_empty_values({
         'status': status,
         'media_ids': media_ids,
         'visibility': visibility,
@@ -191,14 +222,18 @@ def post_status(
         'language': language,
         'scheduled_at': scheduled_at,
         'content_type': content_type,
-        'spoiler_text': spoiler_text
-    }
+        'spoiler_text': spoiler_text,
+    })
 
-    # Strip keys for which value is None
-    # Sending null values doesn't bother Mastodon, but it breaks Pleroma
-    json = {k: v for k, v in json.items() if v is not None}
+    if poll_options:
+        data["poll"] = {
+            "options": poll_options,
+            "expires_in": poll_expires_in,
+            "multiple": poll_multiple,
+            "hide_totals": poll_hide_totals,
+        }
 
-    return http.post(app, user, '/api/v1/statuses', json=json, headers=headers).json()
+    return http.post(app, user, '/api/v1/statuses', json=data, headers=headers)
 
 
 def fetch_status(app, user, id):
@@ -206,7 +241,7 @@ def fetch_status(app, user, id):
     Fetch a single status
     https://docs.joinmastodon.org/methods/statuses/#get
     """
-    return http.get(app, user, f"/api/v1/statuses/{id}").json()
+    return http.get(app, user, f"/api/v1/statuses/{id}")
 
 
 def scheduled_statuses(app, user):
@@ -261,14 +296,14 @@ def translate(app, user, status_id):
     return _status_action(app, user, status_id, 'translate')
 
 
-def context(app, user, status_id):
+def context(app, user, status_id) -> Response:
     url = f"/api/v1/statuses/{status_id}/context"
-    return http.get(app, user, url).json()
+    return http.get(app, user, url)
 
 
-def reblogged_by(app, user, status_id):
+def reblogged_by(app, user, status_id) -> Response:
     url = f"/api/v1/statuses/{status_id}/reblogged_by"
-    return http.get(app, user, url).json()
+    return http.get(app, user, url)
 
 
 def _get_next_path(headers):
@@ -295,6 +330,14 @@ def _notification_timeline_generator(app, user, path, params=None):
         path = _get_next_path(response.headers)
 
 
+def _conversation_timeline_generator(app, user, path, params=None):
+    while path:
+        response = http.get(app, user, path, params)
+        conversation = response.json()
+        yield [c["last_status"] for c in conversation if c["last_status"]]
+        path = _get_next_path(response.headers)
+
+
 def home_timeline_generator(app, user, limit=20):
     path = "/api/v1/timelines/home"
     params = {"limit": limit}
@@ -326,6 +369,19 @@ def notification_timeline_generator(app, user, limit=20):
     return _notification_timeline_generator(app, user, "/api/v1/notifications", params)
 
 
+def conversation_timeline_generator(app, user, limit=20):
+    path = "/api/v1/conversations"
+    params = {"limit": limit}
+    return _conversation_timeline_generator(app, user, path, params)
+
+
+def account_timeline_generator(app: App, user: User, account_name: str, replies=False, reblogs=False, limit=20):
+    account = find_account(app, user, account_name)
+    path = f"/api/v1/accounts/{account['id']}/statuses"
+    params = {"limit": limit, "exclude_replies": not replies, "exclude_reblogs": not reblogs}
+    return _timeline_generator(app, user, path, params)
+
+
 def timeline_list_generator(app, user, list_id, limit=20):
     path = f"/api/v1/timelines/list/{list_id}"
     return _timeline_generator(app, user, path, {'limit': limit})
@@ -351,11 +407,44 @@ def anon_tag_timeline_generator(instance, hashtag, local=False, limit=20):
     return _anon_timeline_generator(instance, path, params)
 
 
-def upload_media(app, user, file, description=None):
-    return http.post(app, user, '/api/v1/media',
-        data={'description': description},
-        files={'file': file}
-    ).json()
+def get_media(app: App, user: User, id: str):
+    return http.get(app, user, f"/api/v1/media/{id}").json()
+
+
+def upload_media(
+    app: App,
+    user: User,
+    media: BinaryIO,
+    description: Optional[str] = None,
+    thumbnail: Optional[BinaryIO] = None,
+):
+    data = drop_empty_values({"description": description})
+
+    # NB: Documentation says that "file" should provide a mime-type which we
+    # don't do currently, but it works.
+    files = drop_empty_values({
+        "file": media,
+        "thumbnail": _add_mime_type(thumbnail)
+    })
+
+    return http.post(app, user, "/api/v2/media", data=data, files=files).json()
+
+
+def _add_mime_type(file):
+    if file is None:
+        return None
+
+    # TODO: mimetypes uses the file extension to guess the mime type which is
+    # not always good enough (e.g. files without extension). python-magic could
+    # be used instead but it requires adding it as a dependency.
+    mime_type = mimetypes.guess_type(file.name)
+
+    if not mime_type:
+        raise ConsoleError(f"Unable guess mime type of '{file.name}'. "
+                           "Ensure the file has the desired extension.")
+
+    filename = path.basename(file.name)
+    return (filename, file, mime_type)
 
 
 def search(app, user, query, resolve=False, type=None):
@@ -363,11 +452,13 @@ def search(app, user, query, resolve=False, type=None):
     Perform a search.
     https://docs.joinmastodon.org/methods/search/#v2
     """
-    return http.get(app, user, "/api/v2/search", {
+    params = drop_empty_values({
         "q": query,
-        "resolve": resolve,
+        "resolve": str_bool(resolve),
         "type": type
-    }).json()
+    })
+
+    return http.get(app, user, "/api/v2/search", params)
 
 
 def follow(app, user, account):
@@ -433,6 +524,10 @@ def unmute(app, user, account):
     return _account_action(app, user, account, 'unmute')
 
 
+def muted(app, user):
+    return _get_response_list(app, user, "/api/v1/mutes")
+
+
 def block(app, user, account):
     return _account_action(app, user, account, 'block')
 
@@ -441,13 +536,12 @@ def unblock(app, user, account):
     return _account_action(app, user, account, 'unblock')
 
 
-def verify_credentials(app, user):
-    return http.get(app, user, '/api/v1/accounts/verify_credentials').json()
+def blocked(app, user):
+    return _get_response_list(app, user, "/api/v1/blocks")
 
 
-def single_status(app, user, status_id):
-    url = f"/api/v1/statuses/{status_id}"
-    return http.get(app, user, url).json()
+def verify_credentials(app, user) -> Response:
+    return http.get(app, user, '/api/v1/accounts/verify_credentials')
 
 
 def get_notifications(app, user, exclude_types=[], limit=20):
@@ -459,6 +553,48 @@ def clear_notifications(app, user):
     http.post(app, user, '/api/v1/notifications/clear')
 
 
-def get_instance(domain, scheme="https"):
-    url = f"{scheme}://{domain}/api/v1/instance"
-    return http.anon_get(url).json()
+def get_instance(base_url: str) -> Response:
+    url = f"{base_url}/api/v1/instance"
+    return http.anon_get(url)
+
+
+def get_lists(app, user):
+    path = "/api/v1/lists"
+    return _get_response_list(app, user, path)
+
+
+def find_list_id(app, user, title):
+    lists = get_lists(app, user)
+    for list_item in lists:
+        if list_item["title"] == title:
+            return list_item["id"]
+    return None
+
+
+def get_list_accounts(app, user, list_id):
+    path = f"/api/v1/lists/{list_id}/accounts"
+    return _get_response_list(app, user, path)
+
+
+def create_list(app, user, title, replies_policy):
+    url = "/api/v1/lists"
+    json = {'title': title}
+    if replies_policy:
+        json['replies_policy'] = replies_policy
+    return http.post(app, user, url, json=json).json()
+
+
+def delete_list(app, user, id):
+    return http.delete(app, user, f"/api/v1/lists/{id}")
+
+
+def add_accounts_to_list(app, user, list_id, account_ids):
+    url = f"/api/v1/lists/{list_id}/accounts"
+    json = {'account_ids': account_ids}
+    return http.post(app, user, url, json=json).json()
+
+
+def remove_accounts_from_list(app, user, list_id, account_ids):
+    url = f"/api/v1/lists/{list_id}/accounts"
+    json = {'account_ids': account_ids}
+    return http.delete(app, user, url, json=json)
