@@ -7,23 +7,16 @@ from getpass import getpass
 from toot import api, config, DEFAULT_INSTANCE, User, App
 from toot.exceptions import ApiError, ConsoleError
 from toot.output import print_out
+from urllib.parse import urlparse
 
 
-def register_app(domain, scheme='https'):
-    print_out("Looking up instance info...")
-    instance = api.get_instance(domain, scheme)
-
-    print_out("Found instance <blue>{}</blue> running Mastodon version <yellow>{}</yellow>".format(
-        instance['title'], instance['version']))
-
+def register_app(domain, base_url):
     try:
         print_out("Registering application...")
-        response = api.create_app(domain, scheme)
+        response = api.create_app(base_url)
     except ApiError:
         raise ConsoleError("Registration failed.")
 
-    base_url = scheme + '://' + domain
-
     app = App(domain, base_url, response['client_id'], response['client_secret'])
     config.save_app(app)
 
@@ -32,20 +25,48 @@ def register_app(domain, scheme='https'):
     return app
 
 
-def create_app_interactive(instance=None, scheme='https'):
-    if not instance:
-        print_out("Choose an instance [<green>{}</green>]: ".format(DEFAULT_INSTANCE), end="")
-        instance = input()
-        if not instance:
-            instance = DEFAULT_INSTANCE
+def create_app_interactive(base_url):
+    if not base_url:
+        print_out(f"Enter instance URL [<green>{DEFAULT_INSTANCE}</green>]: ", end="")
+        base_url = input()
+        if not base_url:
+            base_url = DEFAULT_INSTANCE
 
-    return config.load_app(instance) or register_app(instance, scheme)
+    domain = get_instance_domain(base_url)
+
+    return config.load_app(domain) or register_app(domain, base_url)
+
+
+def get_instance_domain(base_url):
+    print_out("Looking up instance info...")
+
+    instance = api.get_instance(base_url).json()
+
+    print_out(
+        f"Found instance <blue>{instance['title']}</blue> "
+        f"running Mastodon version <yellow>{instance['version']}</yellow>"
+    )
+
+    # Pleroma and its forks return an actual URI here, rather than a
+    # domain name like Mastodon. This is contrary to the spec.¯
+    # in that case, parse out the domain and return it.
+
+    parsed_uri = urlparse(instance["uri"])
+
+    if parsed_uri.netloc:
+        # Pleroma, Akkoma, GotoSocial, etc.
+        return parsed_uri.netloc
+    else:
+        # Others including Mastodon servers
+        return parsed_uri.path
+
+    # NB: when updating to v2 instance endpoint, this field has been renamed to `domain`
 
 
 def create_user(app, access_token):
     # Username is not yet known at this point, so fetch it from Mastodon
     user = User(app.instance, None, access_token)
-    creds = api.verify_credentials(app, user)
+    creds = api.verify_credentials(app, user).json()
 
     user = User(app.instance, creds['username'], access_token)
     config.save_user(user, activate=True)
