@@ -7,9 +7,10 @@ import sys
 from argparse import ArgumentParser, FileType, ArgumentTypeError, Action
 from collections import namedtuple
 from itertools import chain
-from toot import config, commands, CLIENT_NAME, CLIENT_WEBSITE, __version__
+from toot import config, commands, CLIENT_NAME, CLIENT_WEBSITE, __version__, settings
 from toot.exceptions import ApiError, ConsoleError
 from toot.output import print_out, print_err
+from toot.settings import get_setting
 
 VISIBILITY_CHOICES = ["public", "unlisted", "private", "direct"]
 VISIBILITY_CHOICES_STR = ", ".join(f"'{v}'" for v in VISIBILITY_CHOICES)
@@ -105,6 +106,10 @@ DURATION_UNITS = {
 }
 
 
+DURATION_EXAMPLES = """e.g. "1 day", "2 hours 30 minutes", "5 minutes 30
+seconds" or any combination of above. Shorthand: "1d", "2h30m", "5m30s\""""
+
+
 def duration(value: str):
     match = re.match(r"""^
         (([0-9]+)\s*(days|day|d))?\s*
@@ -187,6 +192,11 @@ account_arg = (["account"], {
     "help": "account name, e.g. 'Gargron@mastodon.social'",
 })
 
+optional_account_arg = (["account"], {
+    "nargs": "?",
+    "help": "account name, e.g. 'Gargron@mastodon.social'",
+})
+
 instance_arg = (["-i", "--instance"], {
     "type": str,
     "help": 'mastodon instance to log into e.g. "mastodon.social"',
@@ -223,6 +233,12 @@ tag_arg = (["tag_name"], {
     "help": "tag name, e.g. Caturday, or \"#Caturday\"",
 })
 
+json_arg = (["--json"], {
+    "action": "store_true",
+    "default": False,
+    "help": "print json instead of plaintext",
+})
+
 # Arguments for selecting a timeline (see `toot.commands.get_timeline_generator`)
 common_timeline_args = [
     (["-p", "--public"], {
@@ -234,6 +250,10 @@ common_timeline_args = [
         "type": str,
         "help": "show hashtag timeline (does not require auth)",
     }),
+    (["-a", "--account"], {
+        "type": str,
+        "help": "show timeline for the given account",
+    }),
     (["-l", "--local"], {
         "action": "store_true",
         "default": False,
@@ -285,7 +305,7 @@ AUTH_COMMANDS = [
     Command(
         name="activate",
         description="Switch between logged in accounts.",
-        arguments=[account_arg],
+        arguments=[optional_account_arg],
         require_auth=False,
     ),
     Command(
@@ -348,8 +368,9 @@ AUTH_COMMANDS = [
             }),
             (["--language"], {
                 "type": language,
-                "help": "Default language to use for authored statuses (ISO 6391)."
+                "help": "Default language to use for authored statuses (ISO 639-1)."
             }),
+            json_arg,
         ],
         require_auth=True,
     ),
@@ -375,7 +396,7 @@ READ_COMMANDS = [
     Command(
         name="whoami",
         description="Display logged in user details",
-        arguments=[],
+        arguments=[json_arg],
         require_auth=True,
     ),
     Command(
@@ -385,6 +406,7 @@ READ_COMMANDS = [
             (["account"], {
                 "help": "account name or numeric ID"
             }),
+            json_arg,
         ],
         require_auth=True,
     ),
@@ -419,6 +441,7 @@ READ_COMMANDS = [
                 "nargs": "?",
             }),
             scheme_arg,
+            json_arg,
         ],
         require_auth=False,
     ),
@@ -434,6 +457,7 @@ READ_COMMANDS = [
                 "default": False,
                 "help": "Resolve non-local accounts",
             }),
+            json_arg,
         ],
         require_auth=True,
     ),
@@ -444,6 +468,18 @@ READ_COMMANDS = [
             (["status_id"], {
                 "help": "Show thread for toot.",
             }),
+            json_arg,
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="status",
+        description="Show a single status",
+        arguments=[
+            (["status_id"], {
+                "help": "ID of the status to show.",
+            }),
+            json_arg,
         ],
         require_auth=True,
     ),
@@ -482,6 +518,12 @@ POST_COMMANDS = [
                 "help": "plain-text description of the media for accessibility "
                         "purposes, one per attached media"
             }),
+            (["--thumbnail"], {
+                "action": "append",
+                "type": FileType("rb"),
+                "help": "path to an image file to serve as media thumbnail, "
+                        "one per attached media"
+            }),
             visibility_arg,
             (["-s", "--sensitive"], {
                 "action": 'store_true',
@@ -498,7 +540,7 @@ POST_COMMANDS = [
             }),
             (["-l", "--language"], {
                 "type": language,
-                "help": "ISO 639-2 language code of the toot, to skip automatic detection",
+                "help": "ISO 639-1 language code of the toot, to skip automatic detection",
             }),
             (["-e", "--editor"], {
                 "type": editor,
@@ -514,16 +556,36 @@ POST_COMMANDS = [
             }),
             (["--scheduled-in"], {
                 "type": duration,
-                "help": """Schedule the toot to be posted after a given amount
-                        of time. Examples: "1 day", "2 hours 30 minutes",
-                        "5 minutes 30 seconds" or any combination of above.
-                        Shorthand: "1d", "2h30m", "5m30s". Must be at least 5
+                "help": f"""Schedule the toot to be posted after a given amount
+                        of time, {DURATION_EXAMPLES}. Must be at least 5
                         minutes.""",
             }),
             (["-t", "--content-type"], {
                 "type": str,
                 "help": "MIME type for the status text (not supported on all instances)",
             }),
+            (["--poll-option"], {
+                "action": "append",
+                "type": str,
+                "help": "Possible answer to the poll"
+            }),
+            (["--poll-expires-in"], {
+                "type": duration,
+                "help": f"""Duration that the poll should be open,
+                        {DURATION_EXAMPLES}. Defaults to 24h.""",
+                "default": 24 * 60 * 60,
+            }),
+            (["--poll-multiple"], {
+                "action": "store_true",
+                "default": False,
+                "help": "Allow multiple answers to be selected."
+            }),
+            (["--poll-hide-totals"], {
+                "action": "store_true",
+                "default": False,
+                "help": "Hide vote counts until the poll ends. Defaults to false."
+            }),
+            json_arg,
         ],
         require_auth=True,
     ),
@@ -548,61 +610,61 @@ STATUS_COMMANDS = [
     Command(
         name="delete",
         description="Delete a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="favourite",
         description="Favourite a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="unfavourite",
         description="Unfavourite a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="reblog",
         description="Reblog a status",
-        arguments=[status_id_arg, visibility_arg],
+        arguments=[status_id_arg, visibility_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="unreblog",
         description="Unreblog a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="reblogged_by",
         description="Show accounts that reblogged the status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=False,
     ),
     Command(
         name="pin",
         description="Pin a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="unpin",
         description="Unpin a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="bookmark",
         description="Bookmark a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="unbookmark",
         description="Unbookmark a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, json_arg],
         require_auth=True,
     ),
 ]
@@ -611,65 +673,63 @@ ACCOUNTS_COMMANDS = [
     Command(
         name="follow",
         description="Follow an account",
-        arguments=[
-            account_arg,
-        ],
+        arguments=[account_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="unfollow",
         description="Unfollow an account",
-        arguments=[
-            account_arg,
-        ],
+        arguments=[account_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="following",
-        description="List accounts followed by the given account",
-        arguments=[
-            account_arg,
-        ],
+        description="List accounts followed by the given account, " +
+                    "or your account if no account given",
+        arguments=[optional_account_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="followers",
-        description="List accounts following the given account",
-        arguments=[
-            account_arg,
-        ],
+        description="List accounts following the given account, " +
+                    "or your account if no account given",
+        arguments=[optional_account_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="mute",
         description="Mute an account",
-        arguments=[
-            account_arg,
-        ],
+        arguments=[account_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="unmute",
         description="Unmute an account",
-        arguments=[
-            account_arg,
-        ],
+        arguments=[account_arg, json_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="muted",
+        description="List muted accounts",
+        arguments=[json_arg],
         require_auth=True,
     ),
     Command(
         name="block",
         description="Block an account",
-        arguments=[
-            account_arg,
-        ],
+        arguments=[account_arg, json_arg],
         require_auth=True,
     ),
     Command(
         name="unblock",
         description="Unblock an account",
-        arguments=[
-            account_arg,
-        ],
+        arguments=[account_arg, json_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="blocked",
+        description="List blocked accounts",
+        arguments=[json_arg],
         require_auth=True,
     ),
 ]
@@ -695,6 +755,101 @@ TAG_COMMANDS = [
     ),
 ]
 
+LIST_COMMANDS = [
+    Command(
+        name="lists",
+        description="List all lists",
+        arguments=[],
+        require_auth=True,
+    ),
+    Command(
+        name="list_accounts",
+        description="List the accounts in a list",
+        arguments=[
+            (["--id"], {
+                "type": str,
+                "help": "ID of the list"
+            }),
+            (["title"], {
+                "type": str,
+                "nargs": "?",
+                "help": "title of the list"
+            }),
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="list_create",
+        description="Create a list",
+        arguments=[
+            (["title"], {
+                "type": str,
+                "help": "title of the list"
+            }),
+            (["--replies-policy"], {
+                "type": str,
+                "help": "replies policy: 'followed', 'list', or 'none' (defaults to 'none')"
+            }),
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="list_delete",
+        description="Delete a list",
+        arguments=[
+            (["--id"], {
+                "type": str,
+                "help": "ID of the list"
+            }),
+            (["title"], {
+                "type": str,
+                "nargs": "?",
+                "help": "title of the list"
+            }),
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="list_add",
+        description="Add account to list",
+        arguments=[
+            (["--id"], {
+                "type": str,
+                "help": "ID of the list"
+            }),
+            (["title"], {
+                "type": str,
+                "nargs": "?",
+                "help": "title of the list"
+            }),
+            (["account"], {
+                "type": str,
+                "help": "Account to add"
+            }),
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="list_remove",
+        description="Remove account from list",
+        arguments=[
+            (["--id"], {
+                "type": str,
+                "help": "ID of the list"
+            }),
+            (["title"], {
+                "type": str,
+                "nargs": "?",
+                "help": "title of the list"
+            }),
+            (["account"], {
+                "type": str,
+                "help": "Account to remove"
+            }),
+        ],
+        require_auth=True,
+    ),
+]
 COMMAND_GROUPS = [
     ("Authentication", AUTH_COMMANDS),
     ("TUI", TUI_COMMANDS),
@@ -703,6 +858,7 @@ COMMAND_GROUPS = [
     ("Status", STATUS_COMMANDS),
     ("Accounts", ACCOUNTS_COMMANDS),
     ("Hashtags", TAG_COMMANDS),
+    ("Lists", LIST_COMMANDS),
 ]
 
 COMMANDS = list(chain(*[commands for _, commands in COMMAND_GROUPS]))
@@ -739,12 +895,24 @@ def get_argument_parser(name, command):
     if command.require_auth:
         combined_args += common_auth_args
 
+    defaults = get_setting(f"commands.{name}", dict, {})
+
     for args, kwargs in combined_args:
+        # Set default value from settings if exists
+        default = get_default_value(defaults, args)
+        if default is not None:
+            kwargs["default"] = default
         parser.add_argument(*args, **kwargs)
 
     return parser
 
 
+def get_default_value(defaults, args):
+    # Hacky way to determine command name from argparse args
+    name = args[-1].lstrip("-").replace("-", "_")
+    return defaults.get(name)
+
+
 def run_command(app, user, name, args):
     command = next((c for c in COMMANDS if c.name == name), None)
 
@@ -776,9 +944,8 @@ def run_command(app, user, name, args):
 
 
 def main():
-    # Enable debug logging if --debug is in args
-    if "--debug" in sys.argv:
-        filename = os.getenv("TOOT_LOG_FILE")
+    if settings.get_debug():
+        filename = settings.get_debug_file()
         logging.basicConfig(level=logging.DEBUG, filename=filename)
         logging.getLogger("urllib3").setLevel(logging.INFO)
 
