@@ -3,12 +3,13 @@ import re
 import sys
 import textwrap
 
-from textwrap import wrap
-from toot.tui.utils import parse_datetime
-from wcwidth import wcswidth
-
-from toot.utils import get_text, parse_html
+from functools import lru_cache
+from toot import settings
+from toot.utils import get_text, html_to_paragraphs
+from toot.entities import Account, Instance, Notification, Poll, Status
 from toot.wcstring import wc_wrap
+from typing import Iterable, List
+from wcwidth import wcswidth
 
 
 STYLES = {
@@ -100,6 +101,7 @@ def strip_tags(message):
     return re.sub(STYLE_TAG_PATTERN, "", message)
 
 
+@lru_cache(maxsize=None)
 def use_ansi_color():
     """Returns True if ANSI color codes should be used."""
 
@@ -116,45 +118,44 @@ def use_ansi_color():
     if "--no-color" in sys.argv:
         return False
 
+    # Check in settings
+    color = settings.get_setting("common.color", bool)
+    if color is not None:
+        return color
+
+    # Use color by default
     return True
 
 
-USE_ANSI_COLOR = use_ansi_color()
-
-QUIET = "--quiet" in sys.argv
-
-
 def print_out(*args, **kwargs):
-    if not QUIET:
-        args = [colorize(a) if USE_ANSI_COLOR else strip_tags(a) for a in args]
+    if not settings.get_quiet():
+        args = [colorize(a) if use_ansi_color() else strip_tags(a) for a in args]
         print(*args, **kwargs)
 
 
 def print_err(*args, **kwargs):
     args = [f"<red>{a}</red>" for a in args]
-    args = [colorize(a) if USE_ANSI_COLOR else strip_tags(a) for a in args]
+    args = [colorize(a) if use_ansi_color() else strip_tags(a) for a in args]
     print(*args, file=sys.stderr, **kwargs)
 
 
-def print_instance(instance):
-    print_out(f"<green>{instance['title']}</green>")
-    print_out(f"<blue>{instance['uri']}</blue>")
-    print_out(f"running Mastodon {instance['version']}")
+def print_instance(instance: Instance):
+    print_out(f"<green>{instance.title}</green>")
+    print_out(f"<blue>{instance.uri}</blue>")
+    print_out(f"running Mastodon {instance.version}")
     print_out()
 
-    description = instance.get("description")
-    if description:
-        for paragraph in re.split(r"[\r\n]+", description.strip()):
+    if instance.description:
+        for paragraph in re.split(r"[\r\n]+", instance.description.strip()):
             paragraph = get_text(paragraph)
             print_out(textwrap.fill(paragraph, width=80))
             print_out()
 
-    rules = instance.get("rules")
-    if rules:
+    if instance.rules:
         print_out("Rules:")
-        for ordinal, rule in enumerate(rules):
+        for ordinal, rule in enumerate(instance.rules):
             ordinal = f"{ordinal + 1}."
-            lines = textwrap.wrap(rule["text"], 80 - len(ordinal))
+            lines = textwrap.wrap(rule.text, 80 - len(ordinal))
             first = True
             for line in lines:
                 if first:
@@ -162,26 +163,40 @@ def print_instance(instance):
                     first = False
                 else:
                     print_out(f"{' ' * len(ordinal)} {line}")
+        print_out()
+
+    contact = instance.contact_account
+    if contact:
+        print_out(f"Contact: {contact.display_name} @{contact.acct}")
 
 
-def print_account(account):
-    print_out(f"<green>@{account['acct']}</green> {account['display_name']}")
+def print_account(account: Account):
+    print_out(f"<green>@{account.acct}</green> {account.display_name}")
 
-    note = get_text(account['note'])
-
-    if note:
+    if account.note:
         print_out("")
-        print_out("\n".join(wrap(note)))
+        print_html(account.note)
+
+    since = account.created_at.strftime('%Y-%m-%d')
 
     print_out("")
-    print_out(f"ID: <green>{account['id']}</green>")
-    print_out(f"Since: <green>{account['created_at'][:10]}</green>")
+    print_out(f"ID: <green>{account.id}</green>")
+    print_out(f"Since: <green>{since}</green>")
     print_out("")
-    print_out(f"Followers: <yellow>{account['followers_count']}</yellow>")
-    print_out(f"Following: <yellow>{account['following_count']}</yellow>")
-    print_out(f"Statuses: <yellow>{account['statuses_count']}</yellow>")
+    print_out(f"Followers: <yellow>{account.followers_count}</yellow>")
+    print_out(f"Following: <yellow>{account.following_count}</yellow>")
+    print_out(f"Statuses: <yellow>{account.statuses_count}</yellow>")
+
+    if account.fields:
+        for field in account.fields:
+            name = field.name.title()
+            print_out(f'\n<yellow>{name}</yellow>:')
+            print_html(field.value)
+            if field.verified_at:
+                print_out("<green>✓ Verified</green>")
+
     print_out("")
-    print_out(account["url"])
+    print_out(account.url)
 
 
 HASHTAG_PATTERN = re.compile(r'(?<!\w)(#\w+)\b')
@@ -196,6 +211,11 @@ def print_acct_list(accounts):
         print_out(f"* <green>@{account['acct']}</green> {account['display_name']}")
 
 
+def print_user_list(users):
+    for user in users:
+        print_out(f"* {user}")
+
+
 def print_tag_list(tags):
     if tags:
         for tag in tags:
@@ -204,6 +224,43 @@ def print_tag_list(tags):
         print_out("You're not following any hashtags.")
 
 
+def print_lists(lists):
+    headers = ["ID", "Title", "Replies"]
+    data = [[lst["id"], lst["title"], lst["replies_policy"]] for lst in lists]
+    print_table(headers, data)
+
+
+def print_table(headers: List[str], data: List[List[str]]):
+    widths = [[len(cell) for cell in row] for row in data + [headers]]
+    widths = [max(width) for width in zip(*widths)]
+
+    def style(string, tag):
+        return f"<{tag}>{string}</{tag}>" if tag else string
+
+    def print_row(row, tag=None):
+        for idx, cell in enumerate(row):
+            width = widths[idx]
+            print_out(style(cell.ljust(width), tag), end="")
+            print_out("  ", end="")
+        print_out()
+
+    underlines = ["-" * width for width in widths]
+
+    print_row(headers, "bold")
+    print_row(underlines, "dim")
+
+    for row in data:
+        print_row(row)
+
+
+def print_list_accounts(accounts):
+    if accounts:
+        print_out("Accounts in list</green>:\n")
+        print_acct_list(accounts)
+    else:
+        print_out("This list has no accounts.")
+
+
 def print_search_results(results):
     accounts = results['accounts']
     hashtags = results['hashtags']
@@ -220,20 +277,18 @@ def print_search_results(results):
         print_out("<yellow>Nothing found</yellow>")
 
 
-def print_status(status, width):
-    reblog = status['reblog']
-    content = reblog['content'] if reblog else status['content']
-    media_attachments = reblog['media_attachments'] if reblog else status['media_attachments']
-    in_reply_to = status['in_reply_to_id']
-    poll = reblog.get('poll') if reblog else status.get('poll')
+def print_status(status: Status, width: int = 80):
+    status_id = status.id
+    in_reply_to_id = status.in_reply_to_id
+    reblogged_by = status.account if status.reblog else None
 
-    time = parse_datetime(status['created_at'])
-    time = time.strftime('%Y-%m-%d %H:%M %Z')
+    status = status.original
 
-    username = "@" + status['account']['acct']
+    time = status.created_at.strftime('%Y-%m-%d %H:%M %Z')
+    username = "@" + status.account.acct
     spacing = width - wcswidth(username) - wcswidth(time) - 2
 
-    display_name = status['account']['display_name']
+    display_name = status.account.display_name
     if display_name:
         spacing -= wcswidth(display_name) + 1
 
@@ -244,57 +299,66 @@ def print_status(status, width):
         f"<yellow>{time}</yellow>",
     )
 
-    for paragraph in parse_html(content):
-        print_out("")
-        for line in paragraph:
-            for subline in wc_wrap(line, width):
-                print_out(highlight_hashtags(subline))
+    print_out("")
+    print_html(status.content, width)
 
-    if media_attachments:
+    if status.media_attachments:
         print_out("\nMedia:")
-        for attachment in media_attachments:
-            url = attachment["url"]
+        for attachment in status.media_attachments:
+            url = attachment.url
             for line in wc_wrap(url, width):
                 print_out(line)
 
-    if poll:
-        print_poll(poll)
+    if status.poll:
+        print_poll(status.poll)
 
     print_out()
+
     print_out(
-        f"ID <yellow>{status['id']}</yellow> ",
-        f"↲ In reply to <yellow>{in_reply_to}</yellow> " if in_reply_to else "",
-        f"↻ Reblogged <blue>@{reblog['account']['acct']}</blue> " if reblog else "",
+        f"ID <yellow>{status_id}</yellow> ",
+        f"↲ In reply to <yellow>{in_reply_to_id}</yellow> " if in_reply_to_id else "",
+        f"↻ <blue>@{reblogged_by.acct}</blue> boosted " if reblogged_by else "",
     )
 
 
-def print_poll(poll):
-    print_out()
-    for idx, option in enumerate(poll["options"]):
-        perc = (round(100 * option["votes_count"] / poll["votes_count"])
-            if poll["votes_count"] else 0)
+def print_html(text, width=80):
+    first = True
+    for paragraph in html_to_paragraphs(text):
+        if not first:
+            print_out("")
+        for line in paragraph:
+            for subline in wc_wrap(line, width):
+                print_out(highlight_hashtags(subline))
+        first = False
 
-        if poll["voted"] and poll["own_votes"] and idx in poll["own_votes"]:
+
+def print_poll(poll: Poll):
+    print_out()
+    for idx, option in enumerate(poll.options):
+        perc = (round(100 * option.votes_count / poll.votes_count)
+            if poll.votes_count and option.votes_count is not None else 0)
+
+        if poll.voted and poll.own_votes and idx in poll.own_votes:
             voted_for = " <yellow>✓</yellow>"
         else:
             voted_for = ""
 
-        print_out(f'{option["title"]} - {perc}% {voted_for}')
+        print_out(f'{option.title} - {perc}% {voted_for}')
 
-    poll_footer = f'Poll · {poll["votes_count"]} votes'
+    poll_footer = f'Poll · {poll.votes_count} votes'
 
-    if poll["expired"]:
+    if poll.expired:
         poll_footer += " · Closed"
 
-    if poll["expires_at"]:
-        expires_at = parse_datetime(poll["expires_at"]).strftime("%Y-%m-%d %H:%M")
+    if poll.expires_at:
+        expires_at = poll.expires_at.strftime("%Y-%m-%d %H:%M")
         poll_footer += f" · Closes on {expires_at}"
 
     print_out()
     print_out(poll_footer)
 
 
-def print_timeline(items, width=100):
+def print_timeline(items: Iterable[Status], width=100):
     print_out("─" * width)
     for item in items:
         print_status(item, width)
@@ -309,20 +373,19 @@ notification_msgs = {
 }
 
 
-def print_notification(notification, width=100):
-    account = "{display_name} @{acct}".format(**notification["account"])
-    msg = notification_msgs.get(notification["type"])
+def print_notification(notification: Notification, width=100):
+    account = f"{notification.account.display_name} @{notification.account.acct}"
+    msg = notification_msgs.get(notification.type)
     if msg is None:
         return
 
     print_out("─" * width)
     print_out(msg.format(account=account))
-    status = notification.get("status")
-    if status is not None:
-        print_status(status, width)
+    if notification.status:
+        print_status(notification.status, width)
 
 
-def print_notifications(notifications, width=100):
+def print_notifications(notifications: List[Notification], width=100):
     for notification in notifications:
         print_notification(notification)
     print_out("─" * width)
