@@ -1,9 +1,10 @@
 import logging
+import subprocess
 import urwid
 
 from concurrent.futures import ThreadPoolExecutor
 
-from toot import api, config, __version__
+from toot import api, config, __version__, settings
 from toot.console import get_default_visibility
 from toot.exceptions import ApiError
 
@@ -14,13 +15,16 @@ from .overlays import ExceptionStackTrace, GotoMenu, Help, StatusSource, StatusL
 from .overlays import StatusDeleteConfirmation, Account
 from .poll import Poll
 from .timeline import Timeline
-from .utils import parse_content_links, show_media
+from .utils import get_max_toot_chars, parse_content_links, copy_to_clipboard
 
 logger = logging.getLogger(__name__)
 
 urwid.set_encoding('UTF-8')
 
 
+DEFAULT_MAX_TOOT_CHARS = 500
+
+
 class Header(urwid.WidgetWrap):
     def __init__(self, app, user):
         self.app = app
@@ -72,29 +76,51 @@ class Footer(urwid.Pile):
 
 class TUI(urwid.Frame):
     """Main TUI frame."""
+    loop: urwid.MainLoop
+    screen: urwid.BaseScreen
 
-    @classmethod
-    def create(cls, app, user, args):
+    @staticmethod
+    def create(app, user, args):
         """Factory method, sets up TUI and an event loop."""
+        screen = TUI.create_screen(args)
+        tui = TUI(app, user, screen, args)
+
+        palette = PALETTE.copy()
+        overrides = settings.get_setting("tui.palette", dict, {})
+        for name, styles in overrides.items():
+            palette.append(tuple([name] + styles))
 
-        tui = cls(app, user, args)
         loop = urwid.MainLoop(
             tui,
-            palette=PALETTE,
+            palette=palette,
             event_loop=urwid.AsyncioEventLoop(),
             unhandled_input=tui.unhandled_input,
+            screen=screen,
         )
         tui.loop = loop
 
         return tui
 
-    def __init__(self, app, user, args):
+    @staticmethod
+    def create_screen(args):
+        screen = urwid.raw_display.Screen()
+
+        # Determine how many colors to use
+        default_colors = 1 if args.no_color else 16
+        colors = settings.get_setting("tui.colors", int, default_colors)
+        logger.debug(f"Setting colors to {colors}")
+        screen.set_terminal_properties(colors)
+
+        return screen
+
+    def __init__(self, app, user, screen, args):
         self.app = app
         self.user = user
         self.args = args
         self.config = config.load_config()
 
-        self.loop = None  # set in `create`
+        self.loop = None  # late init, set in `create`
+        self.screen = screen
         self.executor = ThreadPoolExecutor(max_workers=1)
         self.timeline_generator = api.home_timeline_generator(app, user, limit=40)
 
@@ -105,20 +131,23 @@ class TUI(urwid.Frame):
         self.footer.set_status("Loading...")
 
         # Default max status length, updated on startup
-        self.max_toot_chars = 500
+        self.max_toot_chars = DEFAULT_MAX_TOOT_CHARS
 
         self.timeline = None
         self.overlay = None
         self.exception = None
         self.can_translate = False
+        self.account = None
+        self.followed_accounts = []
+        self.media_viewer = settings.get_setting("tui.media_viewer", str)
 
         super().__init__(self.body, header=self.header, footer=self.footer)
 
     def run(self):
         self.loop.set_alarm_in(0, lambda *args: self.async_load_instance())
-        self.loop.set_alarm_in(0, lambda *args: self.async_load_followed_tags())
         self.loop.set_alarm_in(0, lambda *args: self.async_load_timeline(
             is_initial=True, timeline_name="home"))
+        self.loop.set_alarm_in(0, lambda *args: self.async_load_followed_accounts())
         self.loop.run()
         self.executor.shutdown(wait=False)
 
@@ -176,56 +205,7 @@ class TUI(urwid.Frame):
         return future
 
     def connect_default_timeline_signals(self, timeline):
-        def _account(timeline, account_id):
-            self.show_account(account_id)
-
-        def _compose(*args):
-            self.show_compose()
-
-        def _delete(timeline, status):
-            if status.is_mine:
-                self.show_delete_confirmation(status)
-
-        def _reply(timeline, status):
-            self.show_compose(status)
-
-        def _source(timeline, status):
-            self.show_status_source(status)
-
-        def _links(timeline, status):
-            self.show_links(status)
-
-        def _media(timeline, status):
-            self.show_media(status)
-
-        def _menu(timeline, status):
-            self.show_context_menu(status)
-
-        def _poll(timeline, status):
-            self.show_poll(status)
-
-        def _zoom(timeline, status_details):
-            self.show_status_zoom(status_details)
-
-        def _clear(*args):
-            self.clear_screen()
-
-        urwid.connect_signal(timeline, "account", _account)
-        urwid.connect_signal(timeline, "bookmark", self.async_toggle_bookmark)
-        urwid.connect_signal(timeline, "compose", _compose)
-        urwid.connect_signal(timeline, "delete", _delete)
-        urwid.connect_signal(timeline, "favourite", self.async_toggle_favourite)
         urwid.connect_signal(timeline, "focus", self.refresh_footer)
-        urwid.connect_signal(timeline, "media", _media)
-        urwid.connect_signal(timeline, "menu", _menu)
-        urwid.connect_signal(timeline, "poll", _poll)
-        urwid.connect_signal(timeline, "reblog", self.async_toggle_reblog)
-        urwid.connect_signal(timeline, "reply", _reply)
-        urwid.connect_signal(timeline, "source", _source)
-        urwid.connect_signal(timeline, "links", _links)
-        urwid.connect_signal(timeline, "zoom", _zoom)
-        urwid.connect_signal(timeline, "translate", self.async_translate)
-        urwid.connect_signal(timeline, "clear-screen", _clear)
 
     def build_timeline(self, name, statuses, local):
         def _close(*args):
@@ -234,9 +214,6 @@ class TUI(urwid.Frame):
         def _next(*args):
             self.async_load_timeline(is_initial=False)
 
-        def _thread(timeline, status):
-            self.show_thread(status)
-
         def _toggle_save(timeline, status):
             if not timeline.name.startswith("#"):
                 return
@@ -252,12 +229,11 @@ class TUI(urwid.Frame):
             self.loop.set_alarm_in(5, lambda *args: self.footer.clear_message())
             config.save_config(self.config)
 
-        timeline = Timeline(name, statuses, self.can_translate, self.followed_tags)
+        timeline = Timeline(self, name, statuses)
 
         self.connect_default_timeline_signals(timeline)
         urwid.connect_signal(timeline, "next", _next)
         urwid.connect_signal(timeline, "close", _close)
-        urwid.connect_signal(timeline, "thread", _thread)
         urwid.connect_signal(timeline, "save", _toggle_save)
 
         return timeline
@@ -275,19 +251,18 @@ class TUI(urwid.Frame):
 
         # This is pretty fast, so it's probably ok to block while context is
         # loaded, can be made async later if needed
-        context = api.context(self.app, self.user, status.original.id)
+        context = api.context(self.app, self.user, status.original.id).json()
         ancestors = [self.make_status(s) for s in context["ancestors"]]
         descendants = [self.make_status(s) for s in context["descendants"]]
         statuses = ancestors + [status] + descendants
         focus = len(ancestors)
 
-        timeline = Timeline("thread", statuses, self.can_translate,
-                            self.followed_tags, focus, is_thread=True)
+        timeline = Timeline(self, "thread", statuses, focus=focus, is_thread=True)
 
         self.connect_default_timeline_signals(timeline)
         urwid.connect_signal(timeline, "close", _close)
-
         self.body = timeline
+        timeline.refresh_status_details()
         self.refresh_footer(timeline)
 
     def async_load_timeline(self, is_initial, timeline_name=None, local=None):
@@ -331,11 +306,11 @@ class TUI(urwid.Frame):
         See: https://github.com/mastodon/mastodon/issues/19328
         """
         def _load_instance():
-            return api.get_instance(self.app.instance)
+            return api.get_instance(self.app.base_url).json()
 
         def _done(instance):
-            if "max_toot_chars" in instance:
-                self.max_toot_chars = instance["max_toot_chars"]
+            self.max_toot_chars = get_max_toot_chars(instance, DEFAULT_MAX_TOOT_CHARS)
+            logger.info(f"Max toot chars set to: {self.max_toot_chars}")
 
             if "translation" in instance:
                 # instance is advertising translation service
@@ -351,21 +326,20 @@ class TUI(urwid.Frame):
 
         return self.run_in_thread(_load_instance, done_callback=_done)
 
-    def async_load_followed_tags(self):
-        def _load_tag_list():
+    def async_load_followed_accounts(self):
+        def _load_accounts():
             try:
-                return api.followed_tags(self.app, self.user)
+                acct = f'@{self.user.username}@{self.user.instance}'
+                self.account = api.find_account(self.app, self.user, acct)
+                return api.following(self.app, self.user, self.account["id"])
             except ApiError:
                 # not supported by all Mastodon servers so fail silently if necessary
                 return []
 
-        def _done_tag_list(tags):
-            if len(tags) > 0:
-                self.followed_tags = [t["name"] for t in tags]
-            else:
-                self.followed_tags = []
+        def _done_accounts(accounts):
+            self.followed_accounts = {a["acct"] for a in accounts}
 
-        self.run_in_thread(_load_tag_list, done_callback=_done_tag_list)
+        self.run_in_thread(_load_accounts, done_callback=_done_accounts)
 
     def refresh_footer(self, timeline):
         """Show status details in footer."""
@@ -382,11 +356,11 @@ class TUI(urwid.Frame):
         )
 
     def clear_screen(self):
-        self.loop.screen.clear()
+        self.screen.clear()
 
     def show_links(self, status):
-        links = parse_content_links(status.data["content"]) if status else []
-        post_attachments = status.data["media_attachments"] or []
+        links = parse_content_links(status.original.data["content"]) if status else []
+        post_attachments = status.original.data["media_attachments"] or []
         reblog_attachments = (status.data["reblog"]["media_attachments"] if status.data["reblog"] else None) or []
 
         for a in post_attachments + reblog_attachments:
@@ -397,6 +371,8 @@ class TUI(urwid.Frame):
             self.clear_screen()
 
         if links:
+            links = list(set(links))  # deduplicate links
+            links = sorted(links, key=lambda link: link[0])  # sort alphabetically by URL
             sl_widget = StatusLinks(links)
             urwid.connect_signal(sl_widget, "clear-screen", _clear)
             self.open_overlay(
@@ -431,7 +407,9 @@ class TUI(urwid.Frame):
 
     def show_goto_menu(self):
         user_timelines = self.config.get("timelines", {})
-        menu = GotoMenu(user_timelines)
+        user_lists = api.get_lists(self.app, self.user) or []
+
+        menu = GotoMenu(user_timelines, user_lists)
         urwid.connect_signal(menu, "home_timeline",
             lambda x: self.goto_home_timeline())
         urwid.connect_signal(menu, "public_timeline",
@@ -440,12 +418,18 @@ class TUI(urwid.Frame):
             lambda x, local: self.goto_bookmarks())
         urwid.connect_signal(menu, "notification_timeline",
             lambda x, local: self.goto_notifications())
+        urwid.connect_signal(menu, "conversation_timeline",
+            lambda x, local: self.goto_conversations())
+        urwid.connect_signal(menu, "personal_timeline",
+            lambda x, local: self.goto_personal_timeline())
         urwid.connect_signal(menu, "hashtag_timeline",
             lambda x, tag, local: self.goto_tag_timeline(tag, local=local))
+        urwid.connect_signal(menu, "list_timeline",
+            lambda x, list_item: self.goto_list_timeline(list_item))
 
         self.open_overlay(menu, title="Go to", options=dict(
             align="center", width=("relative", 60),
-            valign="middle", height=11 + len(user_timelines),
+            valign="middle", height=18 + len(user_timelines) + len(user_lists),
         ))
 
     def show_help(self):
@@ -466,7 +450,8 @@ class TUI(urwid.Frame):
     def goto_public_timeline(self, local):
         self.timeline_generator = api.public_timeline_generator(
             self.app, self.user, local=local, limit=40)
-        promise = self.async_load_timeline(is_initial=True, timeline_name="public")
+        timeline_name = "local public" if local else "global public"
+        promise = self.async_load_timeline(is_initial=True, timeline_name=timeline_name)
         promise.add_done_callback(lambda *args: self.close_overlay())
 
     def goto_bookmarks(self):
@@ -481,6 +466,15 @@ class TUI(urwid.Frame):
         promise = self.async_load_timeline(is_initial=True, timeline_name="notifications")
         promise.add_done_callback(lambda *args: self.close_overlay())
 
+    def goto_conversations(self):
+        self.timeline_generator = api.conversation_timeline_generator(
+            self.app, self.user, limit=40
+        )
+        promise = self.async_load_timeline(
+            is_initial=True, timeline_name="conversations"
+        )
+        promise.add_done_callback(lambda *args: self.close_overlay())
+
     def goto_tag_timeline(self, tag, local):
         self.timeline_generator = api.tag_timeline_generator(
             self.app, self.user, tag, local=local, limit=40)
@@ -489,10 +483,30 @@ class TUI(urwid.Frame):
         )
         promise.add_done_callback(lambda *args: self.close_overlay())
 
+    def goto_personal_timeline(self):
+        account_name = f"{self.user.username}@{self.user.instance}"
+
+        self.timeline_generator = api.account_timeline_generator(
+            self.app, self.user, account_name, reblogs=True, limit=40)
+        promise = self.async_load_timeline(is_initial=True, timeline_name=f"personal {account_name}")
+        promise.add_done_callback(lambda *args: self.close_overlay())
+
+    def goto_list_timeline(self, list_item):
+        self.timeline_generator = api.timeline_list_generator(
+            self.app, self.user, list_item['id'], limit=40)
+        promise = self.async_load_timeline(
+            is_initial=True, timeline_name=f"\N{clipboard}{list_item['title']}")
+        promise.add_done_callback(lambda *args: self.close_overlay())
+
     def show_media(self, status):
         urls = [m["url"] for m in status.original.data["media_attachments"]]
-        if urls:
-            show_media(urls)
+        if not urls:
+            return
+
+        if self.media_viewer:
+            subprocess.run([self.media_viewer] + urls)
+        else:
+            self.footer.set_error_message("Media viewer not configured")
 
     def show_context_menu(self, status):
         # TODO: show context menu
@@ -510,15 +524,20 @@ class TUI(urwid.Frame):
         urwid.connect_signal(widget, "close", _close)
         urwid.connect_signal(widget, "delete", _delete)
         self.open_overlay(widget, title="Delete status?", options=dict(
-            align="center", width=("relative", 60),
-            valign="middle", height=5,
+            align="center", width=30,
+            valign="middle", height=4,
         ))
 
     def post_status(self, content, warning, visibility, in_reply_to_id):
-        data = api.post_status(self.app, self.user, content,
+        data = api.post_status(
+            self.app,
+            self.user,
+            content,
             spoiler_text=warning,
             visibility=visibility,
-            in_reply_to_id=in_reply_to_id)
+            in_reply_to_id=in_reply_to_id
+        ).json()
+
         status = self.make_status(data)
 
         # TODO: fetch new items from the timeline?
@@ -536,11 +555,9 @@ class TUI(urwid.Frame):
 
     def async_toggle_favourite(self, timeline, status):
         def _favourite():
-            logger.info("Favouriting {}".format(status))
             api.favourite(self.app, self.user, status.id)
 
         def _unfavourite():
-            logger.info("Unfavouriting {}".format(status))
             api.unfavourite(self.app, self.user, status.id)
 
         def _done(loop):
@@ -557,18 +574,16 @@ class TUI(urwid.Frame):
 
     def async_toggle_reblog(self, timeline, status):
         def _reblog():
-            logger.info("Reblogging {}".format(status))
-            api.reblog(self.app, self.user, status.id, visibility=get_default_visibility())
+            api.reblog(self.app, self.user, status.original.id, visibility=get_default_visibility())
 
         def _unreblog():
-            logger.info("Unreblogging {}".format(status))
-            api.unreblog(self.app, self.user, status.id)
+            api.unreblog(self.app, self.user, status.original.id)
 
         def _done(loop):
             # Create a new Status with flipped reblogged flag
             new_data = status.data
-            new_data["reblogged"] = not status.reblogged
             new_status = self.make_status(new_data)
+            new_status.original.reblogged = not status.original.reblogged
             timeline.update_status(new_status)
 
         # Check if status is rebloggable
@@ -579,17 +594,16 @@ class TUI(urwid.Frame):
             return
 
         self.run_in_thread(
-            _unreblog if status.reblogged else _reblog,
+            _unreblog if status.original.reblogged else _reblog,
             done_callback=_done
         )
 
     def async_translate(self, timeline, status):
         def _translate():
-            logger.info("Translating {}".format(status))
-            self.footer.set_message("Translating status {}".format(status.id))
+            self.footer.set_message("Translating status {}".format(status.original.id))
 
             try:
-                response = api.translate(self.app, self.user, status.id)
+                response = api.translate(self.app, self.user, status.original.id)
                 if response["content"]:
                     self.footer.set_message("Status translated")
                 else:
@@ -604,25 +618,23 @@ class TUI(urwid.Frame):
 
         def _done(response):
             if response is not None:
-                status.translation = response["content"]
-                status.translated_from = response["detected_source_language"]
-                status.show_translation = True
+                status.original.translation = response["content"]
+                status.original.translated_from = response["detected_source_language"]
+                status.original.show_translation = True
                 timeline.update_status(status)
 
         # If already translated, toggle showing translation
-        if status.translation:
-            status.show_translation = not status.show_translation
+        if status.original.translation:
+            status.original.show_translation = not status.original.show_translation
             timeline.update_status(status)
         else:
             self.run_in_thread(_translate, done_callback=_done)
 
     def async_toggle_bookmark(self, timeline, status):
         def _bookmark():
-            logger.info("Bookmarking {}".format(status))
             api.bookmark(self.app, self.user, status.id)
 
         def _unbookmark():
-            logger.info("Unbookmarking {}".format(status))
             api.unbookmark(self.app, self.user, status.id)
 
         def _done(loop):
@@ -646,6 +658,12 @@ class TUI(urwid.Frame):
 
         return self.run_in_thread(_delete, done_callback=_done)
 
+    def copy_status(self, status):
+        # TODO: copy a better version of status content
+        # including URLs
+        copy_to_clipboard(self.screen, status.original.data["content"])
+        self.footer.set_message(f"Status {status.original.id} copied")
+
     # --- Overlay handling -----------------------------------------------------
 
     default_overlay_options = dict(
@@ -673,6 +691,38 @@ class TUI(urwid.Frame):
         if self.timeline:
             self.timeline.refresh_status_details()
 
+    def refresh_timeline(self):
+        # No point in refreshing the bookmarks timeline
+        # and we don't have a good way to refresh a
+        # list timeline yet (no reference to list ID kept)
+        if (not self.timeline
+                or self.timeline.name == 'bookmarks'
+                or self.timeline.name.startswith("\N{clipboard}")):
+            return
+
+        if self.timeline.name.startswith("#"):
+            self.timeline_generator = api.tag_timeline_generator(
+                self.app, self.user, self.timeline.name[1:], limit=40)
+        elif self.timeline.name.startswith("\N{clipboard}"):
+            self.timeline_generator = api.tag_timeline_generator(
+                self.app, self.user, self.timeline.name[1:], limit=40)
+        else:
+            if self.timeline.name.endswith("public"):
+                self.timeline_generator = api.public_timeline_generator(
+                    self.app, self.user, local=self.timeline.name.startswith("local"), limit=40)
+            elif self.timeline.name == "notifications":
+                self.timeline_generator = api.notification_timeline_generator(
+                    self.app, self.user, limit=40)
+            elif self.timeline.name == "conversations":
+                self.timeline_generator = api.conversation_timeline_generator(
+                    self.app, self.user, limit=40)
+            else:
+                # default to home timeline
+                self.timeline_generator = api.home_timeline_generator(
+                    self.app, self.user, limit=40)
+
+        self.async_load_timeline(is_initial=True, timeline_name=self.timeline.name)
+
     # --- Keys -----------------------------------------------------------------
 
     def unhandled_input(self, key):
@@ -685,15 +735,13 @@ class TUI(urwid.Frame):
             if not self.overlay:
                 self.show_goto_menu()
 
-        elif key in ('h', 'H'):
+        elif key == '?':
             if not self.overlay:
                 self.show_help()
 
         elif key == ',':
             if not self.overlay:
-                self.timeline_generator = api.home_timeline_generator(
-                    self.app, self.user, limit=40)
-                self.async_load_timeline(is_initial=True, timeline_name=self.timeline.name)
+                self.refresh_timeline()
 
         elif key == 'esc':
             if self.overlay:
